#!/usr/bin/python3
import sys
import os
import subprocess
from datetime import datetime
import tempfile
from audit_common import AuditError, read_audit, next_event, next_msg,\
    check_event, assert_audit_enabled


def enable_file_audit(enable, verbose=True):
    rc = subprocess.call([
        'auditctl',
        '-a' if enable else '-d',
        'always,exit',
        '-F', 'obj_type=faud',
        '-S', 'all',
        '-k', 'parsec-tests'],
        stderr=None if verbose else subprocess.DEVNULL)
    if rc and verbose:
        raise AuditError(
            "Can't {} audit system.".format(
                "start" if enable else "stop"))


def setfaud(options, path):
    rc = subprocess.call([
        'setfaud',
        '-m',
        'o:{}:{}'.format(options, options),
        path])
    if rc:
        raise AuditError("Can't set audit flags to {}.".format(path))


def regtest(name, filename):
    print("Creation of {} event {}...".format(
        name, filename))
    to_check_messages.append(
        "Searching for {} messages within log...".format(
            name))


# ***************************************************************
try:
    assert_audit_enabled()
    # remove old audit rule
    enable_file_audit(False, verbose=False)
except (AuditError, FileNotFoundError) as e:
    print(str(e))
    if os.getuid():
        print("\nPlease, run as root.\n")
    sys.exit(1)

try:
    name = ''
    to_check = []
    to_check_messages = []
    print("Execution of audit system...")
    enable_file_audit(True)
    start = datetime.now()

    print("# Testing audit system")

# Subtest - create create event
    print("Setting audit flags to /tmp dir...")
    setfaud('c', '/tmp')
    with tempfile.NamedTemporaryFile(delete=False) as f:
        name = f.name
    to_check_messages.append(
        "Searching for create messages within log...")
    to_check.append([
        {'type': 'PATH',
         'name': name,
         'nametype': 'CREATE'},
        {'type': 'SYSCALL',
         'syscall': 'open'}])
# Subtest - set flags
    print("Setting audit flags to file {}...".format(name))
    setfaud('oxudnarmc', name)
    to_check_messages.append(
        "Searching for audit on messages within log...")
    to_check.append([
        {'type': 'AVC',
         'fn': 'ch_aud',
         'path': name}])
# Subtest - create open event
    regtest('open', name)
    with open(name):
        pass
    to_check.append([
        {'type': 'PATH',
         'name': name},
        {'type': 'SYSCALL',
         'syscall': 'open'}])
# Subtest - create chmod event
    regtest('chmod', name)
    subprocess.call(['chmod', '+x', name])
    to_check.append([
        {'type': 'PATH',
         'name': name},
        {'type': 'SYSCALL',
         'syscall': 'chmod'}])
# Subtest - create chown event
    regtest('chown', name)
    subprocess.call(['chown', 'root:root', name])
    to_check.append([
        {'type': 'PATH',
         'name': name},
        {'type': 'SYSCALL',
         'syscall': 'chown'}])
# Subtest - create setacl event
    regtest('setacl', name)
    subprocess.call(['setfacl', '-m', 'u:root:r', name])
    to_check.append([
        {'type': 'PATH',
         'name': name},
        {'type': 'AVC',
         'fn': 'acl'}])
# Subtest - create parsec_chmac event
    regtest('mac', name)
    subprocess.call(['pdpl-file', '0:0', name])
    to_check.append([
        {'type': 'PATH',
         'name': name},
        {'type': 'AVC',
         'fn': 'sys_chlbl',
         'object': '0:0:0:0'}])
# Subtest - create exec event
    regtest('exec', name)
    try:
        subprocess.call([name])
    except OSError:
        pass
    to_check.append([
        {'type': 'PATH',
         'name': name},
        {'type': 'SYSCALL',
         'syscall': 'exec'}])
# Subtest - create audit event
    regtest('audit off', name)
    setfaud('u', name)
    to_check.append([
        {'type': 'AVC',
         'fn': 'ch_aud',
         'path': name}])
# Subtest - create unlnik event
    regtest('unlink', name)
    os.unlink(name)
    to_check.append([
        {'type': 'PATH',
         'name': name},
        {'type': 'SYSCALL',
         'syscall': 'unlink'}])

# Subtest - search for events
    end = datetime.now()
    enable_file_audit(False)

    i = 0
    print(to_check_messages[i])
    for event in next_event(read_audit(start, end)):
        if check_event(to_check[i], event):
            i += 1
            if i == len(to_check):
                sys.exit(0)
            print(to_check_messages[i])

    # print(read_audit(start, end))
    print("Incorrect work of audit system")

except AuditError as e:
    print(str(e))

finally:
    if name:
        print("Removing audit flags from /tmp dir...")
        subprocess.call(['setfaud', '-b', '/tmp'])
        try:
            os.unlink(name)
        except FileNotFoundError:
            pass

sys.exit(1)
